<?php

namespace WPRuby_CAA\Core\App\Backend\Endpoints;

use WPRuby_CAA\Core\Constants;
use WPRuby_CAA\Core\Helpers\Helper;

class Get_User_Activity_Endpoint extends Abstract_Endpoint {

	public function callback( $data )
	{
		global $wpdb;
		$user_id = intval($data['user_id']);
		$user = get_user_by('id', $user_id);

		$table_name = Constants::activities_table_name();

		$where = $wpdb->prepare(" WHERE user_id = %d ", $user_id);
		$query = "SELECT * FROM ".$table_name. $where . " ORDER BY created_at desc";
		$results = $wpdb->get_results($query);

		$activities = [];
		$dateTimeFormat = Helper::date_time_format();
		foreach ($results as $result) {
			$activities[] = [
				'action' => ucwords(str_replace('_', ' ', $result->action)),
				'created_at_human' => human_time_diff(strtotime($result->created_at)),
				'created_at' => date($dateTimeFormat, strtotime($result->created_at)),
				'user_text' => sprintf('%s %s', $user->first_name, $user->last_name),
				'user_uri' => sprintf('%suser-edit.php?user_id=%s', get_admin_url(), $user_id),
				'user_avatar' => get_avatar_url($user_id),
				'ip_address' => $result->ip_address,
				'ip_address_uri' => sprintf('https://ipinfo.io/%s', $result->ip_address),
				'type' => ucwords(str_replace('_', ' ', $result->object_type)),
				'action_icon' => $this->get_type_icon($result->action),
				'title_uri' => $this->get_title_uri($result->object_type, $result->object_id),
				'title' => $this->get_title($result->action, $result->object_title),
			];
		}

		$this->output(['activities' => $activities]);

	}


	public function action() {
		return 'user_activities';
	}

	private function get_title_uri( $object_type, $object_id )
	{
		$link = get_permalink($object_id);
		if($link) {
			return $link;
		}

		return null;
	}

	private function get_title( $action, $object_title )
	{
		switch ($action) {
			case 'user_role_changed':
				return 'New Role: ' . $object_title;
		}

		return $object_title;
	}

	private function get_type_icon( $action ) {
		switch ($action) {
			case 'user_logged_out':
				return 'exit';
			case 'user_deleted':
			case 'user_role_changed':
			case 'profile_updated':
				return 'admin-users';
			case 'attachment_added':
			case 'attachment_updated':
			case 'attachment_deleted':
				return 'media-archive';
			case 'theme_deleted':
			case 'theme_customizer_saved':
			case 'theme_switched':
				return 'art';
			case 'plugin_deactivated':
			case 'plugin_deleted':
				return 'admin-plugins';
			case 'plugin_activated':
				return 'plugins-checked';
			case 'content_exported':
				return 'database-export';
			default:
				return 'clock';
		}

	}
}
