<?php

namespace WPRuby_CAA\Core\Features\Plugins;

use WPRuby_CAA\Core\Helpers\Helper;

class WooCommerce extends Plugin {


	public function name(): string {
		return 'WooCommerce';
	}

	public function key(): string {
		return 'woocommerce';
	}

	public function is_installed(): bool {
		return class_exists( 'woocommerce' );
	}

	public function filter()
	{
		global $current_tab;
		$this->block_tab_access();

		if (isset($items['settings']) && count($items['settings']) > 0) {
			add_filter('woocommerce_settings_tabs_array', [$this, 'remove_settings_tabs'], 999);
		}

		if (isset($items['status']) && count($items['status']) > 0) {
			add_filter('woocommerce_admin_status_tabs', [$this, 'remove_status_tabs'], 999);
		}

		if (isset($items['reports']) && count($items['reports']) > 0) {
			add_filter('woocommerce_admin_reports', [$this, 'remove_reports_tabs'], 999);
		}


	}

	private function block_tab_access()
	{
		global $current_tab;


		if (isset($this->items_to_block['settings']) && in_array($current_tab, $this->items_to_block['settings'])) {
			Helper::block_access();
		}

		if (isset($this->items_to_block['status'])) {
			$this->block_status_tab_access($this->items_to_block['status']);
		}

		if (isset($this->items_to_block['reports'])) {
			$this->block_reports_tab_access($this->items_to_block['reports']);
		}

	}

	private function block_reports_tab_access(array $items)
	{
		$current_page = ! empty( $_REQUEST['page'] ) ? sanitize_title( $_REQUEST['page'] ) : null;
		$current_tab = ! empty( $_REQUEST['tab'] ) ? sanitize_title( $_REQUEST['tab'] ) : null;

		if (strtolower($current_page) !== 'wc-reports' ) return;
		if ($current_tab === null) return;

		if (in_array($current_tab, $items)) {
			Helper::block_access();
		}
	}

	private function block_status_tab_access(array $items)
	{
		$current_page = ! empty( $_REQUEST['page'] ) ? sanitize_title( $_REQUEST['page'] ) : null;
		$current_tab = ! empty( $_REQUEST['tab'] ) ? sanitize_title( $_REQUEST['tab'] ) : null;

		if (strtolower($current_page) !== 'wc-status' ) return;
		if ($current_tab === null) return;

		if (in_array($current_tab, $items)) {
			Helper::block_access();
		}
	}

	public function remove_status_tabs($tabs) {
		return $this->remove_tabs('status', $tabs);
	}

	public function remove_reports_tabs($tabs) {
		return $this->remove_tabs('reports', $tabs);
	}

	public function remove_settings_tabs($tabs) {
		return $this->remove_tabs('settings', $tabs);
	}

	private function remove_tabs($type, $tabs)
	{
		if(!isset($this->items_to_block[$type])) return $tabs;

		foreach ($this->items_to_block[$type] as $item) {
			if (isset($tabs[$item])) {
				unset($tabs[$item]);
			}
		}
		return $tabs;
	}



	public function menu(): array {

		return [

			'settings' => [
				'name' => 'Settings',
				'menu' => [
					[
						'title' => 'Products',
						'slug'  => 'products'
					],
					[
						'title' => 'Tax',
						'slug'  => 'tax'
					],
					[
						'title' => 'Shipping',
						'slug'  => 'shipping'
					],
					[
						'title' => 'Payments',
						'slug'  => 'checkout'
					],
					[
						'title' => 'Accounts & Privacy',
						'slug'  => 'account'
					],
					[
						'title' => 'Emails',
						'slug'  => 'email'
					],
					[
						'title' => 'Integration',
						'slug'  => 'integration'
					],
					[
						'title' => 'Advanced',
						'slug'  => 'advanced'
					],
				],
			],
			'reports'  => [
				'name' => 'Reports',
				'menu' => [
					[
						'title' => 'Customers',
						'slug'  => 'customers'
					],
					[
						'title' => 'Stock',
						'slug'  => 'stock'
					],
					[
						'title' => 'Taxes',
						'slug'  => 'taxes'
					],
				],
			],
			'status'   => [
				'name' => 'Status',
				'menu' => [
					[
						'title' => 'Tools',
						'slug'  => 'tools'
					],
					[
						'title' => 'Logs',
						'slug'  => 'logs'
					],
					[
						'title' => 'Scheduled Actions',
						'slug'  => 'action-scheduler'
					],
				],
			],
		];
	}
}
